@@ -18,15 +18,28 @@
 #include "parent_pipe_fd.h"
 #include "ctr_exit.h"
 #include "close_fds.h"
+#include "seccomp_notify.h"
 #include "runtime_args.h"
 
-#include <sys/prctl.h>
 #include <sys/stat.h>
 #include <locale.h>
 
+static void disconnect_std_streams(int dev_null_r, int dev_null_w)
+{
+	if (dup2(dev_null_r, STDIN_FILENO) < 0)
+		pexit("Failed to dup over stdin");
+	if (dup2(dev_null_w, STDOUT_FILENO) < 0)
+		pexit("Failed to dup over stdout");
+	if (dup2(dev_null_w, STDERR_FILENO) < 0)
+		pexit("Failed to dup over stderr");
+}
+
+#define DEFAULT_UMASK 0022
+
 int main(int argc, char *argv[])
 {
 	setlocale(LC_ALL, "");
+	umask(DEFAULT_UMASK);
 	_cleanup_gerror_ GError *err = NULL;
 	char buf[BUF_SIZE];
 	int num_read;
@@ -41,7 +54,7 @@ int main(int argc, char *argv[])
 
 	process_cli();
 
-	attempt_oom_adjust("-1000");
+	attempt_oom_adjust(-1000);
 
 	/* ignoring SIGPIPE prevents conmon from being spuriously killed */
 	signal(SIGPIPE, SIG_IGN);
@@ -113,13 +126,8 @@ int main(int argc, char *argv[])
 	/* Disconnect stdio from parent. We need to do this, because
 	   the parent is waiting for the stdout to end when the intermediate
 	   child dies */
-	if (dup2(dev_null_r, STDIN_FILENO) < 0)
-		pexit("Failed to dup over stdin");
-	if (dup2(dev_null_w, STDOUT_FILENO) < 0)
-		pexit("Failed to dup over stdout");
-	if (dup2(dev_null_w, STDERR_FILENO) < 0)
-		pexit("Failed to dup over stderr");
-
+	if (!logging_is_passthrough())
+		disconnect_std_streams(dev_null_r, dev_null_w);
 	/* Create a new session group */
 	setsid();
 
@@ -127,12 +135,13 @@ int main(int argc, char *argv[])
 	 * Set self as subreaper so we can wait for container process
 	 * and return its exit code.
 	 */
-	int ret = prctl(PR_SET_CHILD_SUBREAPER, 1, 0, 0, 0);
+	int ret = set_subreaper(true);
 	if (ret != 0) {
 		pexit("Failed to set as subreaper");
 	}
 
 	_cleanup_free_ char *csname = NULL;
+	_cleanup_free_ char *seccomp_listener = NULL;
 	int workerfd_stdin = -1;
 	int workerfd_stdout = -1;
 	int workerfd_stderr = -1;
@@ -167,12 +176,16 @@ int main(int argc, char *argv[])
 
 		mainfd_stdout = fds[0];
 		workerfd_stdout = fds[1];
+	}
 
-		/* now that we've set mainfd_stdout, we can register the ctrl_winsz_cb
-		 * if we didn't set it here, we'd risk attempting to run ioctl on
-		 * a negative fd, and fail to resize the window */
-		if (winsz_fd_r >= 0)
-			g_unix_fd_add(winsz_fd_r, G_IO_IN, ctrl_winsz_cb, NULL);
+	if (opt_seccomp_notify_socket != NULL) {
+#ifdef USE_SECCOMP
+		pexit("seccomp support not present");
+#else
+		if (opt_seccomp_notify_plugins == NULL)
+			pexit("seccomp notify socket specified without any plugin");
+		seccomp_listener = setup_seccomp_socket(opt_seccomp_notify_socket);
+#endif
 	}
 
 	/* We always create a stderr pipe, because that way we can capture
@@ -187,14 +200,14 @@ int main(int argc, char *argv[])
 
 	/* Setup endpoint for attach */
 	_cleanup_free_ char *attach_symlink_dir_path = NULL;
-	if (opt_bundle_path != NULL) {
+	if (opt_bundle_path != NULL && !logging_is_passthrough()) {
 		attach_symlink_dir_path = setup_attach_socket();
 		dummyfd = setup_terminal_control_fifo();
 		setup_console_fifo();
 
 		if (opt_attach) {
 			ndebug("sending attach message to parent");
-			write_sync_fd(attach_pipe_fd, 0, NULL);
+			write_or_close_sync_fd(&attach_pipe_fd, 0, NULL);
 			ndebug("sent attach message to parent");
 		}
 	}
@@ -216,32 +229,37 @@ int main(int argc, char *argv[])
 	if (create_pid < 0) {
 		pexit("Failed to fork the create command");
 	} else if (!create_pid) {
-		if (prctl(PR_SET_PDEATHSIG, SIGKILL) < 0)
+		if (set_pdeathsig(SIGKILL) < 0)
 			_pexit("Failed to set PDEATHSIG");
 		if (sigprocmask(SIG_SETMASK, &oldmask, NULL) < 0)
 			_pexit("Failed to unblock signals");
 
-		if (workerfd_stdin < 0)
-			workerfd_stdin = dev_null_r;
-		if (dup2(workerfd_stdin, STDIN_FILENO) < 0)
-			_pexit("Failed to dup over stdin");
-		if (workerfd_stdin != dev_null_r && fchmod(STDIN_FILENO, 0777) < 0)
-			nwarn("Failed to chown stdin");
+		if (!logging_is_passthrough()) {
+			/*
+			 * EINVAL indicates the type of file descriptor used is not supporting fchmod(2) on the given platform.
+			 * Only more unusual cases are logged.
+			 */
+			if (workerfd_stdin < 0)
+				workerfd_stdin = dev_null_r;
+			if (dup2(workerfd_stdin, STDIN_FILENO) < 0)
+				_pexit("Failed to dup over stdin");
+			if (workerfd_stdin != dev_null_r && fchmod(STDIN_FILENO, 0777) < 0 && errno != EINVAL)
+				nwarn("Failed to chmod stdin");
 
-		if (workerfd_stdout < 0)
-			workerfd_stdout = dev_null_w;
-		if (dup2(workerfd_stdout, STDOUT_FILENO) < 0)
-			_pexit("Failed to dup over stdout");
-		if (workerfd_stdout != dev_null_w && fchmod(STDOUT_FILENO, 0777) < 0)
-			nwarn("Failed to chown stdout");
-
-		if (workerfd_stderr < 0)
-			workerfd_stderr = workerfd_stdout;
-		if (dup2(workerfd_stderr, STDERR_FILENO) < 0)
-			_pexit("Failed to dup over stderr");
-		if (workerfd_stderr != dev_null_w && fchmod(STDERR_FILENO, 0777) < 0)
-			nwarn("Failed to chown stderr");
+			if (workerfd_stdout < 0)
+				workerfd_stdout = dev_null_w;
+			if (dup2(workerfd_stdout, STDOUT_FILENO) < 0)
+				_pexit("Failed to dup over stdout");
+			if (workerfd_stdout != dev_null_w && fchmod(STDOUT_FILENO, 0777) < 0 && errno != EINVAL)
+				nwarn("Failed to chmod stdout");
 
+			if (workerfd_stderr < 0)
+				workerfd_stderr = workerfd_stdout;
+			if (dup2(workerfd_stderr, STDERR_FILENO) < 0)
+				_pexit("Failed to dup over stderr");
+			if (workerfd_stderr != dev_null_w && fchmod(STDERR_FILENO, 0777) < 0 && errno != EINVAL)
+				nwarn("Failed to chmod stderr");
+		}
 		/* If LISTEN_PID env is set, we need to set the LISTEN_PID
 		   it to the new child process */
 		char *listenpid = getenv("LISTEN_PID");
@@ -276,11 +294,14 @@ int main(int argc, char *argv[])
 		}
 
 		// We don't want runc to be unkillable so we reset the oom_score_adj back to 0
-		attempt_oom_adjust("0");
+		reset_oom_adjust();
 		execv(g_ptr_array_index(runtime_argv, 0), (char **)runtime_argv->pdata);
 		exit(127);
 	}
 
+	if (logging_is_passthrough())
+		disconnect_std_streams(dev_null_r, dev_null_w);
+
 	if ((signal(SIGTERM, on_sig_exit) == SIG_ERR) || (signal(SIGQUIT, on_sig_exit) == SIG_ERR)
 	    || (signal(SIGINT, on_sig_exit) == SIG_ERR))
 		pexit("Failed to register the signal handler");
@@ -293,18 +314,15 @@ int main(int argc, char *argv[])
 	_cleanup_hashtable_ GHashTable *pid_to_handler = g_hash_table_new(g_int_hash, g_int_equal);
 	g_hash_table_insert(pid_to_handler, (pid_t *)&create_pid, runtime_exit_cb);
 
-	/*
-	 * Glib does not support SIGCHLD so use SIGUSR1 with the same semantic.  We will
-	 * catch SIGCHLD and raise(SIGUSR1) in the signal handler.
-	 */
+	/* Register a handler via signalfd for handling SIGCHLD */
 	struct pid_check_data data = {
 		.pid_to_handler = pid_to_handler,
 		.exit_status_cache = NULL,
 	};
-	g_unix_signal_add(SIGUSR1, on_sigusr1_cb, &data);
-
-	if (signal(SIGCHLD, on_sigchld) == SIG_ERR)
-		pexit("Failed to set handler for SIGCHLD");
+	int signal_fd = get_signal_descriptor();
+	if (signal_fd < 0)
+		pexit("Failed to create signalfd");
+	int signal_fd_tag = g_unix_fd_add(signal_fd, G_IO_IN, on_signalfd_cb, &data);
 
 	if (opt_exit_command)
 		atexit(do_exit_command);
@@ -319,6 +337,9 @@ int main(int argc, char *argv[])
 	if (workerfd_stderr > -1)
 		close(workerfd_stderr);
 
+	if (seccomp_listener != NULL)
+		g_unix_fd_add(seccomp_socket_fd, G_IO_IN, seccomp_accept_cb, csname);
+
 	if (csname != NULL) {
 		g_unix_fd_add(console_socket_fd, G_IO_IN, terminal_accept_cb, csname);
 		/* Process any SIGCHLD we may have missed before the signal handler was in place.  */
@@ -358,7 +379,7 @@ int main(int argc, char *argv[])
 				if (opt_exec && container_status > 0) {
 					to_report = -1 * container_status;
 				}
-				write_sync_fd(sync_pipe_fd, to_report, buf);
+				write_or_close_sync_fd(&sync_pipe_fd, to_report, buf);
 			}
 		}
 		nexitf("Failed to create container: exit status %d", get_exit_status(runtime_status));
@@ -385,9 +406,11 @@ int main(int argc, char *argv[])
 	 * Thus, if we are legacy and we are exec, skip this write.
 	 */
 	if ((opt_api_version >= 1 || !opt_exec) && sync_pipe_fd >= 0)
-		write_sync_fd(sync_pipe_fd, container_pid, NULL);
+		write_or_close_sync_fd(&sync_pipe_fd, container_pid, NULL);
 
+#ifdef __linux__
 	setup_oom_handling(container_pid);
+#endif
 
 	if (mainfd_stdout >= 0) {
 		g_unix_fd_add(mainfd_stdout, G_IO_IN, stdio_cb, GINT_TO_POINTER(STDOUT_PIPE));
@@ -437,7 +460,9 @@ int main(int argc, char *argv[])
 		g_main_loop_run(main_loop);
 	}
 
+#ifdef __linux__
 	check_cgroup2_oom();
+#endif
 
 	/* Drain stdout and stderr only if a timeout doesn't occur */
 	if (!timed_out)
@@ -466,6 +491,10 @@ int main(int argc, char *argv[])
 		exit_status = get_exit_status(container_status);
 	}
 
+	/* Close down the signalfd */
+	g_source_remove(signal_fd_tag);
+	close(signal_fd);
+
 	/*
 	 * Podman injects some fd's into the conmon process so that exposed ports are kept busy while
 	 * the container runs.  Close them before we notify the container exited, so that they can be
@@ -492,10 +521,12 @@ int main(int argc, char *argv[])
 		if (!g_file_set_contents(exit_file_path, status_str, -1, &err))
 			nexitf("Failed to write %s to exit file: %s", status_str, err->message);
 	}
+	if (seccomp_listener != NULL)
+		unlink(seccomp_listener);
 
 	/* Send the command exec exit code back to the parent */
 	if (opt_exec && sync_pipe_fd >= 0)
-		write_sync_fd(sync_pipe_fd, exit_status, exit_message);
+		write_or_close_sync_fd(&sync_pipe_fd, exit_status, exit_message);
 
 	if (attach_symlink_dir_path != NULL && unlink(attach_symlink_dir_path) == -1 && errno != ENOENT)
 		pexit("Failed to remove symlink for attach socket directory");
